﻿using System.Collections.Generic;
using System.Linq;

namespace vamtaco
{
	public class PubesClippingReducer : MVRScript
	{
        private DAZCharacterSelector _character;
        private List<PubesNoClipClothingItem> _noClipClothingItems;
        private List<string> _hiddenHairUids;

        public override void Init()
        {
            if (containingAtom.type != "Person")
            {
                SuperController.LogMessage("Pubes Clipping Reducer (Disabled): This plugin is designed to be used on a person atom.");
                enabled = false;
                return;
            }

            _noClipClothingItems = new List<PubesNoClipClothingItem>();
            _hiddenHairUids = new List<string>();

            var geometry = containingAtom.GetStorableByID("geometry");
            _character = geometry as DAZCharacterSelector;

            //load supported clothing
            foreach (var clothing in _character.clothingItems.Where(c => c.tagsArray.Contains("HidePubes")))
			{
				LoadNoClipClothingItem(clothing.displayName);
			}
		}

        void Start()
        {
            //if any clothing is active, hide pubes
            if (_noClipClothingItems.Any(o => o.ClothingItem.isActiveAndEnabled))
            {
                HidePubes();
            }
        }

        void Update()
        {
            foreach (var c in _noClipClothingItems)
            {
                c.EvaluateStateChange();
            }
        }

        public void OnClothingItemStateChanged(DAZClothingItem clothingItem)
        {
            //item is now active so hide pubes
            if (clothingItem.isActiveAndEnabled)
            {
                var clothSimControlChildren = clothingItem.GetComponentsInChildren<ClothSimControl>();
                bool anyUndressed = clothSimControlChildren.Any(o => o.clothSettings.BreakEnabled);

                if (anyUndressed)
                {
                    ShowPubes();
                    return;
                }

                HidePubes();
                return;
            }

            ShowPubes();
        }

        private void HidePubes()
        {
            _hiddenHairUids = _character.hairItems.Where(h => h.active == true && h.tags.Contains("Genital")).Select(h => h.uid).ToList();
            _hiddenHairUids.ForEach(hid => _character.SetActiveHairItem(hid, false));
        }

        private void ShowPubes()
        {
            _hiddenHairUids.ForEach(hid => _character.SetActiveHairItem(hid, true));
            _hiddenHairUids.Clear();
        }

        private void LoadNoClipClothingItem(string name)
        {
            var clothing = _character.clothingItems.SingleOrDefault(c => c.displayName == name);
            if (clothing != null)
            {
                var ncci = new PubesNoClipClothingItem(clothing);
                ncci.OnStateChanged += new PubesNoClipClothingItem.OnStateChangedEvent(OnClothingItemStateChanged);
                _noClipClothingItems.Add(ncci);
            }
        }
    }

    public class PubesNoClipClothingItem
    {
        public string Name { get { return ClothingItem.displayName; } }

        public DAZClothingItem ClothingItem { get; }

        public delegate void OnStateChangedEvent(DAZClothingItem clothingItem);
        public OnStateChangedEvent OnStateChanged;

        private bool _previousState;
        private bool _previousUndressState;
        private ClothSimControl[] _clothSimControls;

        public PubesNoClipClothingItem(DAZClothingItem clothingItem)
        {
            ClothingItem = clothingItem;
            _previousState = clothingItem.isActiveAndEnabled;
            _clothSimControls = clothingItem.GetComponentsInChildren<ClothSimControl>();
            _previousUndressState = _clothSimControls.Any(o => o.clothSettings.BreakEnabled);
        }

        public void EvaluateStateChange()
        {
            if (ClothingItem == null)
                return;

            //clothing item state has changed
            bool anyUndressed = _clothSimControls.Any(o => o.clothSettings.BreakEnabled);

            if (ClothingItem.isActiveAndEnabled != _previousState
                || anyUndressed != _previousUndressState)
            {
                _previousState = ClothingItem.isActiveAndEnabled;
                _previousUndressState = anyUndressed;
                OnStateChanged?.Invoke(ClothingItem);
            }
        }
    }
}
